@@ -197,7 +197,8 @@ module Puppet
       :http_pool => proc {
         require 'puppet/network/http'
         Puppet::Network::HTTP::NoCachePool.new
-      }
+      },
+      :ssl_host => proc { Puppet::SSL::Host.localhost },
     }
   end
 
@@ -273,6 +274,7 @@ end
 #  effort, but I think we should strive for it and revisit this at some point.  --cprice 2012-03-16
 
 require 'puppet/indirector'
+require 'puppet/compilable_resource_type'
 require 'puppet/type'
 require 'puppet/resource'
 require 'puppet/parser'
