@@ -1,10 +1,10 @@
 require 'pathname'
-require_relative 'hiera_interpolate'
+require 'puppet/pops/lookup/interpolation'
 
 module Puppet::DataProviders
   class HieraConfig
     include Puppet::Plugins::DataProviders
-    include HieraInterpolate
+    include Puppet::Pops::Lookup::Interpolation
 
     DEFAULT_CONFIG = {
       'version' => 4,
@@ -27,8 +27,6 @@ module Puppet::DataProviders
         Hash[struct.map { |k,v| [k.to_s, symkeys_to_string(v)] }]
       when Array
         struct.map { |v| symkeys_to_string(v) }
-      when nil
-        {}
       else
         struct
       end
@@ -44,7 +42,7 @@ module Puppet::DataProviders
       hierarchy_elem_type_v2 = hierarchy_elem_type_base + ',paths=>Array[String[1]]}]'
       hierarchy_elem_type_v3 = hierarchy_elem_type_base + '}]'
 
-      Puppet::Pops::Types::TypeParser.new.parse('Struct[{'\
+      Puppet::Pops::Types::TypeParser.singleton.parse('Struct[{'\
         'version=>Integer[4],'\
         "hierarchy=>Optional[Array[Variant[#{hierarchy_elem_type_v1},#{hierarchy_elem_type_v2},#{hierarchy_elem_type_v3}]]],"\
         'datadir=>Optional[String[1]]}]')
