@@ -40,6 +40,8 @@ module Puppet::Environments
     end
 
     def clear_all
+      root = Puppet.lookup(:root_environment) { nil }
+      root.instance_variable_set(:@static_catalogs, nil) unless root.nil?
     end
   end
 
@@ -110,7 +112,7 @@ module Puppet::Environments
     def get_conf(name)
       env = get(name)
       if env
-        Puppet::Settings::EnvironmentConf.static_for(env)
+        Puppet::Settings::EnvironmentConf.static_for(env, 0, Puppet[:static_catalogs])
       else
         nil
       end
@@ -145,8 +147,10 @@ module Puppet::Environments
     include EnvironmentLoader
 
     def initialize(environment_dir, global_module_path)
-      @environment_dir = environment_dir
-      @global_module_path = global_module_path
+      @environment_dir =  Puppet::FileSystem.expand_path(environment_dir)
+      @global_module_path = global_module_path ?
+        global_module_path.map { |p| Puppet::FileSystem.expand_path(p) } :
+        nil
     end
 
     # Generate an array of directory loaders from a path string.
