@@ -6,13 +6,13 @@ Puppet::Parser::Functions::newfunction(
   :arity => 2,
   :doc => <<-DOC
 Scans a string and returns an array of one or more converted values based on the given format string.
-See the documenation of Ruby's String#scanf method for details about the supported formats (which
+See the documentation of Ruby's String#scanf method for details about the supported formats (which
 are similar but not identical to the formats used in Puppet's `sprintf` function.)
 
 This function takes two mandatory arguments: the first is the string to convert, and the second is
-the format string. The result of the scan is an array, with each sucessfully scanned and transformed value.
-The scanning stops if a scan is unsuccesful, and the scanned result up to that point is returned. If there
-was no succesful scan, the result is an empty array.
+the format string. The result of the scan is an array, with each successfully scanned and transformed value.
+The scanning stops if a scan is unsuccessful, and the scanned result up to that point is returned. If there
+was no successful scan, the result is an empty array.
 
    "42".scanf("%i")
 
