@@ -1,12 +1,14 @@
+module Puppet::Pops
+module Parser
 # Helper class that keeps track of where line breaks are located and can answer questions about positions.
 #
-class Puppet::Pops::Parser::Locator
+class Locator
   # Creates, or recreates a Locator. A Locator is created if index is not given (a scan is then
   # performed of the given source string.
   #
   def self.locator(string, file, index = nil, char_offsets = false)
     if char_offsets
-      LocatorForChars.new(string, file, index);
+      LocatorForChars.new(string, file, index)
     else
       Locator19.new(string, file, index)
     end
@@ -73,14 +75,14 @@ class Puppet::Pops::Parser::Locator
   #
   # Note that leading_offset must be the same for all lines and measured in characters.
   #
-  class SubLocator < Puppet::Pops::Parser::Locator
+  class SubLocator < Locator
     attr_reader :locator
     attr_reader :leading_line_count
     attr_reader :leading_offset
     attr_reader :leading_line_offset
 
     def self.sub_locator(string, file, leading_line_count, leading_offset, leading_line_offset)
-      self.new(Puppet::Pops::Parser::Locator.locator(string, file),
+      self.new(Locator.locator(string, file),
         leading_line_count,
         leading_offset,
         leading_line_offset)
@@ -130,7 +132,7 @@ class Puppet::Pops::Parser::Locator
 
   private
 
-  class AbstractLocator < Puppet::Pops::Parser::Locator
+  class AbstractLocator < Locator
     attr_accessor :line_index
     attr_accessor :string
     attr_accessor :prev_offset
@@ -203,6 +205,15 @@ class Puppet::Pops::Parser::Locator
       return low
     end
 
+    def hash
+      [string, file, line_index].hash
+    end
+
+    # Equal method needed by serializer to perform tabulation
+    def eql?(o)
+      self.class == o.class && string == o.string && file == o.file && line_index == o.line_index
+    end
+
     # Common impl for 18 and 19 since scanner is byte based
     def compute_line_index
       scanner = StringScanner.new(string)
@@ -260,6 +271,26 @@ class Puppet::Pops::Parser::Locator
   # strings are frozen).
   #
   class Locator19 < AbstractLocator
+    include Types::PuppetObject
+
+    def self._ptype
+      @type
+    end
+
+    # The Locator is not part of the Ecore model so no ObjectType is automatically inferred. Instead the
+    # type is explicitly added here.
+    # TODO: LocatorForChars is never added. It looks like it could be removed (remnant from Ruby 1.8 compatibility?)
+    # @api private
+    def self.register_ptype(loader, ir)
+      @type = Pcore::create_object_type(loader, ir, self, 'Puppet::AST::Locator', 'Any',
+        'string' => Types::PStringType::DEFAULT,
+        'file' => Types::PStringType::DEFAULT,
+        'line_index' => {
+          Types::KEY_TYPE => Types::POptionalType.new(Types::PArrayType.new(Types::PIntegerType::DEFAULT)),
+          Types::KEY_VALUE => nil
+        }
+      ).resolve(Types::TypeParser.singleton, loader)
+    end
 
     # Returns the offset on line (first offset on a line is 0).
     # Ruby 19 is multibyte but has no character position methods, must use byteslice
@@ -286,3 +317,5 @@ class Puppet::Pops::Parser::Locator
     end
   end
 end
+end
+end
