@@ -53,6 +53,11 @@ class ResourceTypeImpl
     )
   end
 
+  def eql?(other)
+    other.is_a?(Puppet::CompilableResourceType) && @name == other.name
+  end
+  alias == eql?
+
   # Compares this type against the given _other_ (type) and returns -1, 0, or +1 depending on the order.
   # @param other [Object] the object to compare against (produces nil, if not kind of Type}
   # @return [-1, 0, +1, nil] produces -1 if this type is before the given _other_ type, 0 if equals, and 1 if after.
@@ -138,7 +143,7 @@ class ResourceTypeImpl
       when 0
         # TechDebt: The case of specifying title patterns when having no name vars is unspecified behavior in puppet
         # Here it is silently ignored.
-        []
+        nil
       when 1
         if @title_pattners_hash.nil?
           [ [ /(.*)/m, [ [@key_attributes.first] ] ] ]
@@ -166,6 +171,10 @@ class ResourceTypeImpl
     false
   end
 
+  def is_capability?
+    false
+  end
+
   # Answers if the parameter name is a parameter/attribute of this type
   # This is part of the Puppet::Type API
   # Check if used when compiling (it is triggered in an apply)
