@@ -312,6 +312,8 @@ class TypeCalculator
         infer_set_Array(o)
       when Hash
         infer_set_Hash(o)
+      when Semantic::Version
+        infer_set_Version(o)
       else
         infer_set_Object(o)
     end
@@ -429,7 +431,7 @@ class TypeCalculator
 
     if t1.is_a?(PVariantType) && t2.is_a?(PVariantType)
       # The common type is one that complies with either set
-      return PVariantType.new(t1.types | t2.types)
+      return PVariantType.maybe_create(t1.types | t2.types)
     end
 
     if t1.is_a?(PRegexpType) && t2.is_a?(PRegexpType)
@@ -552,7 +554,14 @@ class TypeCalculator
 
   # @api private
   def infer_Object(o)
-    PRuntimeType.new(:ruby, o.class.name)
+    if o.is_a?(PuppetObject)
+      o._ptype
+    else
+      name = o.class.name
+      ir = Loaders.implementation_registry
+      type = ir.nil? ? nil : ir.type_for_module(name)
+      type.nil? ? PRuntimeType.new(:ruby, name) : type
+    end
   end
 
   # The type of all types is PType
@@ -634,6 +643,11 @@ class TypeCalculator
     end
   end
 
+  # @api private
+  def infer_Sensitive(o)
+    PSensitiveType.new(infer(o.unwrap))
+  end
+
   # @api private
   def infer_TrueClass(o)
     PBooleanType::DEFAULT
@@ -652,7 +666,7 @@ class TypeCalculator
     # A mapping must be made to empty string. A nil value will result in an error later
     title = o.title
     title = '' if :undef == title
-    PType.new(PResourceType.new(o.type.to_s.downcase, title))
+    PType.new(PResourceType.new(o.type.to_s, title))
   end
 
   # @api private
@@ -664,6 +678,16 @@ class TypeCalculator
     end
   end
 
+  # @api private
+  def infer_Version(o)
+    PSemVerType::DEFAULT
+  end
+
+  # @api private
+  def infer_VersionRange(o)
+    PSemVerRangeType::DEFAULT
+  end
+
   # @api private
   def infer_Hash(o)
     if o.empty?
@@ -699,12 +723,17 @@ class TypeCalculator
     elsif o.keys.all? {|k| PStringType::NON_EMPTY.instance?(k) }
       PStructType.new(o.each_pair.map { |k,v| PStructElement.new(PStringType.new(size_as_type(k), [k]), infer_set(v)) })
     else
-      ktype = PVariantType.new(o.keys.map {|k| infer_set(k) })
-      etype = PVariantType.new(o.values.map {|e| infer_set(e) })
+      ktype = PVariantType.maybe_create(o.keys.map {|k| infer_set(k) })
+      etype = PVariantType.maybe_create(o.values.map {|e| infer_set(e) })
       PHashType.new(unwrap_single_variant(ktype), unwrap_single_variant(etype), size_as_type(o))
     end
   end
 
+  # @api private
+  def infer_set_Version(o)
+    PSemVerType.new([Semantic::VersionRange.new(o, o)])
+  end
+
   def unwrap_single_variant(possible_variant)
     if possible_variant.is_a?(PVariantType) && possible_variant.types.size == 1
       possible_variant.types[0]
