@@ -310,7 +310,7 @@ class Checker4_0 < Evaluator::LiteralEvaluator
   end
 
   # Only used for function names, grammar should not be able to produce something faulty, but
-  # check anyway if model is created programatically (it will fail in transformation to AST for sure).
+  # check anyway if model is created programmatically (it will fail in transformation to AST for sure).
   def check_NamedAccessExpression(o)
     name = o.right_expr
     unless name.is_a? Model::QualifiedName
@@ -349,7 +349,7 @@ class Checker4_0 < Evaluator::LiteralEvaluator
   # for 'class', 'define', and function
   def check_NamedDefinition(o)
     top(o.eContainer, o)
-    if o.name !~ Patterns::CLASSREF
+    if o.name !~ Patterns::CLASSREF_DECL
       acceptor.accept(Issues::ILLEGAL_DEFINITION_NAME, o, {:name=>o.name})
     end
     internal_check_reserved_type_name(o, o.name)
@@ -358,6 +358,9 @@ class Checker4_0 < Evaluator::LiteralEvaluator
 
   def check_TypeAlias(o)
     top(o.eContainer, o)
+    if o.name !~ Patterns::CLASSREF_EXT_DECL
+      acceptor.accept(Issues::ILLEGAL_DEFINITION_NAME, o, {:name=>o.name})
+    end
     internal_check_reserved_type_name(o, o.name)
     internal_check_type_ref(o, o.type_expr)
   end
