@@ -1,6 +1,6 @@
 require 'puppet'
 require 'puppet/provider/nameservice'
-require 'plist'
+require 'puppet/util/plist' if Puppet.features.cfpropertylist?
 require 'fileutils'
 
 class Puppet::Provider::NameService::DirectoryService < Puppet::Provider::NameService
@@ -18,8 +18,8 @@ class Puppet::Provider::NameService::DirectoryService < Puppet::Provider::NameSe
   commands :dscl => "/usr/bin/dscl"
   commands :dseditgroup => "/usr/sbin/dseditgroup"
   commands :sw_vers => "/usr/bin/sw_vers"
-  commands :plutil => '/usr/bin/plutil'
   confine :operatingsystem => :darwin
+  confine :feature         => :cfpropertylist
   defaultfor :operatingsystem => :darwin
 
 
@@ -63,7 +63,7 @@ class Puppet::Provider::NameService::DirectoryService < Puppet::Provider::NameSe
   def self.instances
     # JJM Class method that provides an array of instance objects of this
     #     type.
-    # JJM: Properties are dependent on the Puppet::Type we're managine.
+    # JJM: Properties are dependent on the Puppet::Type we're managing.
     type_property_array = [:name] + @resource_type.validproperties
 
     # Create a new instance of this Puppet::Type for each object present
@@ -97,7 +97,7 @@ class Puppet::Provider::NameService::DirectoryService < Puppet::Provider::NameSe
   end
 
   def self.parse_dscl_plist_data(dscl_output)
-    Plist.parse_xml(dscl_output)
+    Puppet::Util::Plist.parse_plist(dscl_output)
   end
 
   def self.generate_attribute_hash(input_hash, *type_properties)
@@ -126,7 +126,7 @@ class Puppet::Provider::NameService::DirectoryService < Puppet::Provider::NameSe
 
     # NBK: need to read the existing password here as it's not actually
     # stored in the user record. It is stored at a path that involves the
-    # UUID of the user record for non-Mobile local acccounts.
+    # UUID of the user record for non-Mobile local accounts.
     # Mobile Accounts are out of scope for this provider for now
     attribute_hash[:password] = self.get_password(attribute_hash[:guid], attribute_hash[:name]) if @resource_type.validproperties.include?(:password) and Puppet.features.root?
     attribute_hash
@@ -160,15 +160,15 @@ class Puppet::Provider::NameService::DirectoryService < Puppet::Provider::NameSe
   def self.get_exec_preamble(ds_action, resource_name = nil)
     # JJM 2007-07-24
     #     DSCL commands are often repetitive and contain the same positional
-    #     arguments over and over. See http://developer.apple.com/documentation/Porting/Conceptual/PortingUnix/additionalfeatures/chapter_10_section_9.html
+    #     arguments over and over. See https://developer.apple.com/documentation/Porting/Conceptual/PortingUnix/additionalfeatures/chapter_10_section_9.html
     #     for an example of what I mean.
     #     This method spits out proper DSCL commands for us.
     #     We EXPECT name to be @resource[:name] when called from an instance object.
 
     command_vector = [ command(:dscl), "-plist", "." ]
 
-    # JJM: The actual action to perform.  See "man dscl"
-    #      Common actiosn: -create, -delete, -merge, -append, -passwd
+    # JJM: The actual action to perform. See "man dscl".
+    #      Common actions: -create, -delete, -merge, -append, -passwd
     command_vector << ds_action
     # JJM: get_ds_path will spit back "Users" or "Groups",
     # etc...  Depending on the Puppet::Type of our self.
@@ -186,98 +186,83 @@ class Puppet::Provider::NameService::DirectoryService < Puppet::Provider::NameSe
     # 10.7 uses salted SHA512 password hashes which are 128 characters plus
     # an 8 character salt. Previous versions used a SHA1 hash padded with
     # zeroes. If someone attempts to use a password hash that worked with
-    # a previous version of OX X, we will fail early and warn them.
+    # a previous version of OS X, we will fail early and warn them.
     if password_hash.length != 136
       fail("OS X 10.7 requires a Salted SHA512 hash password of 136 characters. \
            Please check your password and try again.")
     end
 
-    if Puppet::FileSystem.exist?("#{users_plist_dir}/#{resource_name}.plist")
+    plist_file = "#{users_plist_dir}/#{resource_name}.plist"
+    if Puppet::FileSystem.exist?(plist_file)
       # If a plist already exists in /var/db/dslocal/nodes/Default/users, then
       # we will need to extract the binary plist from the 'ShadowHashData'
       # key, log the new password into the resultant plist's 'SALTED-SHA512'
       # key, and then save the entire structure back.
-      users_plist = Plist::parse_xml(plutil( '-convert', 'xml1', '-o', '/dev/stdout', \
-                                     "#{users_plist_dir}/#{resource_name}.plist"))
+      users_plist = Puppet::Util::Plist.read_plist_file(plist_file)
 
-      # users_plist['ShadowHashData'][0].string is actually a binary plist
+      # users_plist['ShadowHashData'][0] is actually a binary plist
       # that's nested INSIDE the user's plist (which itself is a binary
       # plist). If we encounter a user plist that DOESN'T have a
       # ShadowHashData field, create one.
       if users_plist['ShadowHashData']
-        password_hash_plist = users_plist['ShadowHashData'][0].string
-        converted_hash_plist = convert_binary_to_xml(password_hash_plist)
+        password_hash_plist = users_plist['ShadowHashData'][0]
+        converted_hash_plist = convert_binary_to_hash(password_hash_plist)
       else
-        users_plist['ShadowHashData'] = [StringIO.new]
-        converted_hash_plist = {'SALTED-SHA512' => StringIO.new}
+        users_plist['ShadowHashData'] = ''
+        converted_hash_plist = {'SALTED-SHA512' => ''}
       end
 
-      # converted_hash_plist['SALTED-SHA512'].string expects a Base64 encoded
+      # converted_hash_plist['SALTED-SHA512'] expects a Base64 encoded
       # string. The password_hash provided as a resource attribute is a
       # hex value. We need to convert the provided hex value to a Base64
       # encoded string to nest it in the converted hash plist.
-      converted_hash_plist['SALTED-SHA512'].string = \
+      converted_hash_plist['SALTED-SHA512'] = \
         password_hash.unpack('a2'*(password_hash.size/2)).collect { |i| i.hex.chr }.join
 
       # Finally, we can convert the nested plist back to binary, embed it
       # into the user's plist, and convert the resultant plist back to
       # a binary plist.
-      changed_plist = convert_xml_to_binary(converted_hash_plist)
-      users_plist['ShadowHashData'][0].string = changed_plist
-      Plist::Emit.save_plist(users_plist, "#{users_plist_dir}/#{resource_name}.plist")
-      plutil('-convert', 'binary1', "#{users_plist_dir}/#{resource_name}.plist")
+      changed_plist = convert_hash_to_binary(converted_hash_plist)
+      users_plist['ShadowHashData'][0] = changed_plist
+      Puppet::Util::Plist.write_plist_file(users_plist, plist_file, :binary)
     end
   end
 
   def self.get_password(guid, username)
-    if Puppet::FileSystem.exist?("#{users_plist_dir}/#{username}.plist")
+    plist_file = "#{users_plist_dir}/#{username}.plist"
+    if Puppet::FileSystem.exist?(plist_file)
       # If a plist exists in /var/db/dslocal/nodes/Default/users, we will
       # extract the binary plist from the 'ShadowHashData' key, decode the
       # salted-SHA512 password hash, and then return it.
-      users_plist = Plist::parse_xml(plutil('-convert', 'xml1', '-o', '/dev/stdout', "#{users_plist_dir}/#{username}.plist"))
+      users_plist = Puppet::Util::Plist.read_plist_file(plist_file)
+
       if users_plist['ShadowHashData']
-        # users_plist['ShadowHashData'][0].string is actually a binary plist
+        # users_plist['ShadowHashData'][0] is actually a binary plist
         # that's nested INSIDE the user's plist (which itself is a binary
         # plist).
-        password_hash_plist = users_plist['ShadowHashData'][0].string
-        converted_hash_plist = convert_binary_to_xml(password_hash_plist)
+        password_hash_plist = users_plist['ShadowHashData'][0]
+        converted_hash_plist = convert_binary_to_hash(password_hash_plist)
 
-        # converted_hash_plist['SALTED-SHA512'].string is a Base64 encoded
+        # converted_hash_plist['SALTED-SHA512'] is a Base64 encoded
         # string. The password_hash provided as a resource attribute is a
         # hex value. We need to convert the Base64 encoded string to a
         # hex value and provide it back to Puppet.
-        password_hash = converted_hash_plist['SALTED-SHA512'].string.unpack("H*")[0]
+        password_hash = converted_hash_plist['SALTED-SHA512'].unpack("H*")[0]
         password_hash
       end
     end
   end
 
-  # This method will accept a hash that has been returned from Plist::parse_xml
-  # and convert it to a binary plist (string value).
-  def self.convert_xml_to_binary(plist_data)
-    Puppet.debug('Converting XML plist to binary')
-    Puppet.debug('Executing: \'plutil -convert binary1 -o - -\'')
-    IO.popen('plutil -convert binary1 -o - -', 'r+') do |io|
-      io.write plist_data.to_plist
-      io.close_write
-      @converted_plist = io.read
-    end
-    @converted_plist
+  # This method will accept a hash and convert it to a binary plist (string value).
+  def self.convert_hash_to_binary(plist_data)
+    Puppet.debug('Converting plist hash to binary')
+    Puppet::Util::Plist.dump_plist(plist_data, :binary)
   end
 
-  # This method will accept a binary plist (as a string) and convert it to a
-  # hash via Plist::parse_xml.
-  def self.convert_binary_to_xml(plist_data)
-    Puppet.debug('Converting binary plist to XML')
-    Puppet.debug('Executing: \'plutil -convert xml1 -o - -\'')
-    IO.popen('plutil -convert xml1 -o - -', 'r+') do |io|
-      io.write plist_data
-      io.close_write
-      @converted_plist = io.read
-    end
-    Puppet.debug('Converting XML values to a hash.')
-    @plist_hash = Plist::parse_xml(@converted_plist)
-    @plist_hash
+  # This method will accept a binary plist (as a string) and convert it to a hash.
+  def self.convert_binary_to_hash(plist_data)
+    Puppet.debug('Converting binary plist to hash')
+    Puppet::Util::Plist.parse_plist(plist_data)
   end
 
   # Unlike most other *nixes, OS X doesn't provide built in functionality
@@ -333,7 +318,7 @@ class Puppet::Provider::NameService::DirectoryService < Puppet::Provider::NameSe
     exec_arg_vector << ns_to_ds_attribute_map[:guid]
     begin
       guid_output = execute(exec_arg_vector)
-      guid_plist = Plist.parse_xml(guid_output)
+      guid_plist = Puppet::Util::Plist.parse_plist(guid_output)
       # Although GeneratedUID like all DirectoryService values can be multi-valued
       # according to the schema, in practice user accounts cannot have multiple UUIDs
       # otherwise Bad Things Happen, so we just deal with the first value.
